/*----- PROTECTED REGION ID(E2Quadrupole.h) ENABLED START -----*/
/* clang-format on */
//=============================================================================
//
// file :        E2Quadrupole.h
//
// description : Include file for the E2Quadrupole class
//
// project :     Elettra 2.0 pure corrector magnet device
//
// This file is part of Tango device class.
//
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
//
//
// Elettra - Sincrotrone Trieste S.c.p.A.
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#ifndef E2Quadrupole_H
#define E2Quadrupole_H

#include <tango/tango.h>
#include <Quadrupole.h>
/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	E2Quadrupole.h

#ifdef TANGO_LOG
	// cppTango after c934adea (Merge branch 'remove-cout-definition' into 'main', 2022-05-23)
	// nothing to do
#else
	// cppTango 9.3-backports and older
	#define TANGO_LOG       cout
	#define TANGO_LOG_INFO  cout2
	#define TANGO_LOG_DEBUG cout3
#endif // TANGO_LOG

/**
 *  E2Quadrupole class description:
 *    Elettra 2.0 quadrupole magnet device
 *    Use calibration table handle current/strgenght converstions.
 *    The magnet device is connected to a power supply tango device , real or simulated.
 *    
 *    Based on ESRF EBSmagnet concepts and libraries.
 */


namespace E2Quadrupole_ns
{
/*----- PROTECTED REGION ID(E2Quadrupole::Additional Class Declarations) ENABLED START -----*/
/* clang-format on */
//	Additional Class Declarations
const double default_beam_energy = 2.4e9; // default electron  beam energy in GeV
const double E0 =  510998.94999999995; // electron rest mass in eV
const double c_light = 299792458.0; // speed of light  m/s
//class Corrector;
class CurrentEventCallback;
/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	E2Quadrupole::Additional Class Declarations

class E2Quadrupole : public TANGO_BASE_CLASS
{

/*----- PROTECTED REGION ID(E2Quadrupole::Data Members) ENABLED START -----*/
/* clang-format on */
//	Add your own data members
	MagnetModel::Quadrupole* quadrupole;  //magnet model
	double magnet_rigidity;
	double magnet_rigidity_inv;
	Tango::DeviceProxy *current_dev;
	CurrentEventCallback *current_cb;
	int current_event_id;
	int current_set_event_id;
	int state_event_id;
	Tango::WAttribute *cursetAtt;
	Tango::WAttribute *strengthAtt;
	Tango::WAttribute *correctionAtt;
	Tango::WAttribute *resonanceAtt;
	std::vector<double> set_current; //the actual set point of the current (from the power supply)  used for strength calculations
	std::vector<double> set_strength; //the strength set point
	bool design_strength_initialized; //true whe design strenfght has been called at least one , first call is to init valude from DB
public:
	omni_mutex iomutex; //guard for event and write interference
	Tango::DevDouble wValCorrStrength; //last CorrectionStrength write value
	Tango::DevDouble wValResStrength;  //last ResonanceStrength write value
/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	E2Quadrupole::Data Members

//	Device property data members
public:
	//	CurrentDevice:	power supply tango device name
	std::string	currentDevice;
	//	Model:	corrector model (part number)
	std::string	model;
	//	SerialNumber:	serial number of the magnet
	std::string	serialNumber;
	//	MagnetName:	Name of magnet
	std::string	magnetName;
	//	CalibrationFileName:	name of calibration table file name
	std::string	calibrationFileName;
	//	CalibrationPath:	path of calibration and parameter files
	std::string	calibrationPath;
	//	ParameterFileName:	name of auxiliary parametr file name
	//  (EBSmagnet compatibility)
	std::string	parameterFileName;

	bool	mandatoryNotDefined;

//	Attribute data members
public:
	Tango::DevDouble	*attr_Strength_read;
	Tango::DevDouble	*attr_CorrectionStrength_read;
	Tango::DevDouble	*attr_ResonanceStrength_read;
	Tango::DevDouble	*attr_Current_read;
	Tango::DevDouble	*attr_BeamEnergy_read;
	Tango::DevDouble	*attr_DesignStrength_read;

//	Constructors and destructors
public:
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device Name
	 */
	E2Quadrupole(Tango::DeviceClass *cl,std::string &s);
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device Name
	 */
	E2Quadrupole(Tango::DeviceClass *cl,const char *s);
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device name
	 *	@param d	Device description.
	 */
	E2Quadrupole(Tango::DeviceClass *cl,const char *s,const char *d);
	/**
	 * The device object destructor.
	 */
	~E2Quadrupole();


//	Miscellaneous methods
public:
	/*
	 *	will be called at device destruction or at init command.
	 */
	void delete_device();
	/*
	 *	Initialize the device
	 */
	virtual void init_device();
	/*
	 *	Read the device properties from database
	 */
	void get_device_property();
	/*
	 *	Always executed method before execution command method.
	 */
	virtual void always_executed_hook();

	/*
	 *	Check if mandatory property has been set
	 */
	 void check_mandatory_property(Tango::DbDatum &class_prop, Tango::DbDatum &dev_prop);

//	Attribute methods
public:
	//--------------------------------------------------------
	/*
	 *	Method     : E2Quadrupole::read_attr_hardware()
	 *	Description: Hardware acquisition for attributes.
	 */
	//--------------------------------------------------------
	virtual void read_attr_hardware(std::vector<long> &attr_list);
	//--------------------------------------------------------
	/*
	 *	Method     : E2Quadrupole::write_attr_hardware()
	 *	Description: Hardware writing for attributes.
	 */
	//--------------------------------------------------------
	virtual void write_attr_hardware(std::vector<long> &attr_list);

/**
 *	Attribute Strength related methods
 *	Description: quadrupole strength 1/m^3 ( m-3)
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_Strength(Tango::Attribute &attr);
	virtual void write_Strength(Tango::WAttribute &attr);
	virtual bool is_Strength_allowed(Tango::AttReqType type);
/**
 *	Attribute CorrectionStrength related methods
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_CorrectionStrength(Tango::Attribute &attr);
	virtual void write_CorrectionStrength(Tango::WAttribute &attr);
	virtual bool is_CorrectionStrength_allowed(Tango::AttReqType type);
/**
 *	Attribute ResonanceStrength related methods
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_ResonanceStrength(Tango::Attribute &attr);
	virtual void write_ResonanceStrength(Tango::WAttribute &attr);
	virtual bool is_ResonanceStrength_allowed(Tango::AttReqType type);
/**
 *	Attribute Current related methods
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_Current(Tango::Attribute &attr);
	virtual void write_Current(Tango::WAttribute &attr);
	virtual bool is_Current_allowed(Tango::AttReqType type);
/**
 *	Attribute BeamEnergy related methods
 *	Description: Electron beam energy in eV. Used to calculate magnetic rigidity in strength calculations.
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_BeamEnergy(Tango::Attribute &attr);
	virtual void write_BeamEnergy(Tango::WAttribute &attr);
	virtual bool is_BeamEnergy_allowed(Tango::AttReqType type);
/**
 *	Attribute DesignStrength related methods
 *	Description: quadrupole design strength 1/m^3 ( m-3). The strenght reqiured by the nominal optic
 *               The attribute is memorized , expert only.
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_DesignStrength(Tango::Attribute &attr);
	virtual void write_DesignStrength(Tango::WAttribute &attr);
	virtual bool is_DesignStrength_allowed(Tango::AttReqType type);


	//--------------------------------------------------------
	/**
	 *	Method     : E2Quadrupole::add_dynamic_attributes()
	 *	Description: Add dynamic attributes if any.
	 */
	//--------------------------------------------------------
	void add_dynamic_attributes();




//	Command related methods
public:
	/**
	 *	Command Reset related method
	 *	Description: reset the corrector
	 *
	 */
	virtual void reset();
	virtual bool is_Reset_allowed(const CORBA::Any &any);
	/**
	 *	Command ResetResonanceStrength related method
	 *	Description: Transfer ResonanceStranght to CorrectionStrength and set ResonanceStrength to 0. Strngth is not modified
	 *
	 */
	virtual void reset_resonance_strength();
	virtual bool is_ResetResonanceStrength_allowed(const CORBA::Any &any);
	/**
	 *	Command ApplyDesignStrength related method
	 *	Description: Apply the memorized DesignStrength strenght to the magnet.
	 *
	 */
	virtual void apply_design_strength();
	virtual bool is_ApplyDesignStrength_allowed(const CORBA::Any &any);


	//--------------------------------------------------------
	/**
	 *	Method     : E2Quadrupole::add_dynamic_commands()
	 *	Description: Add dynamic commands if any.
	 */
	//--------------------------------------------------------
	void add_dynamic_commands();

/*----- PROTECTED REGION ID(E2Quadrupole::Additional Method prototypes) ENABLED START -----*/
/* clang-format on */
//	Additional Method prototypes
	void calc_rigidity(const double& energy);
	void set_current_write(const double& curr);
	void update_values_write(void); //update internal calculated values when current write value is updated;
	void update_values_read(void); //update internal calculated values when current read value is updated;
	void update_values_state(void); //update internal calculated values when power supply state is updated;
	void init_strengths(void); // initialize strengths from power supply values
	void subscribe_events(void); //subcribe events to power supply
	//void delayed_subscribe(int delayedMs);
/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	E2Quadrupole::Additional Method prototypes
};

/*----- PROTECTED REGION ID(E2Quadrupole::Additional Classes Definitions) ENABLED START -----*/
/* clang-format on */
//	Additional Classes Definitions
class CurrentEventCallback : public Tango::CallBack
{
public:
	CurrentEventCallback(E2Quadrupole*);
	void push_event(Tango::EventData* ev);
protected:
	E2Quadrupole* e2_quad;
};
/* clang-format off */
/*----- PROTECTED REGION END -----*/	//	E2Quadrupole::Additional Classes Definitions

}	//	End of namespace

#endif   //	E2Quadrupole_H
